__AUTHOR__ = '吴子豪 / Vortez Wohl'
__EMAIL__ = 'vortez.wohl@gmail.com'
__GITHUB__ = 'https://github.com/vortezwohl'
__BLOG__ = 'https://vortezwohl.github.io'

from .template import *
from .util import *



def json_to_markdown(json_data: dict | list, indent: int = 0) -> str:
    md = []
    current_indent = '  ' * max(indent - 1, 0)
    next_indent = '  ' * max(indent, 0)
    if isinstance(json_data, dict):
        for key, value in json_data.items():
            if indent == 0:
                md.append(f"## {key}")
                md.append(json_to_markdown(value, indent + 1))
            else:
                next_line = f'{json_to_markdown(value, indent + 1).strip()} '
                if next_line.startswith('-') or next_line[0].isdigit():
                    md.append(f"{current_indent}- **{key}**:")
                    md.append(f'{next_indent}{next_line}')
                else:
                    md.append(f"{current_indent}- **{key}**: {next_line}")
    elif isinstance(json_data, list):
        for i, item in enumerate(json_data):
            item_str = json_to_markdown(item, indent + 1).strip()
            md.append(f"{current_indent}{i + 1}. {item_str}")
    else:
        if json_data is None:
            return "null"
        return str(json_data).lower() if isinstance(json_data, bool) else str(json_data)
    return '\n'.join(md)


def json_replace(json_data: dict | list, _old: str, _new: str) -> dict | list | str:
    if isinstance(json_data, dict):
        for key, value in json_data.items():
            json_data[key] = json_replace(value, _old, _new)
    elif isinstance(json_data, list):
        for i, item in enumerate(json_data):
            json_data[i] = json_replace(item, _old, _new)
    elif isinstance(json_data, str):
        return json_data.replace(_old, _new)
    return json_data



import json
import re
import time

from prompt4py.template.base_template import BaseTemplate
from prompt4py.util.json2markdown import json_to_markdown, json_replace

_VAR_LEFT = '{{'
_VAR_RIGHT = '}}'
_VAR_PATTERN = re.compile(r'{{(\w+)}}')


class GeneralTemplate(BaseTemplate):
    def __init__(self):
        super().__init__()

    def render(self, markdown: bool = True, **kwargs) -> str:
        _prompt_object = {
            '_timestamp'.upper(): f'[{time.perf_counter()}]',
            'role'.upper(): self.role,
            'objective'.upper(): self.objective,
            'instruction'.upper(): self.instruction,
            'constraint'.upper(): self.constraint,
            'capability'.upper(): self.capability,
            'context'.upper(): self.context,
            'output_language'.upper(): self.output_language,
            'output_datatype'.upper(): self.output_dtype,
            'output_format'.upper(): self.output_format,
            'output_example'.upper(): json.dumps(self.output_example, ensure_ascii=False)
            if (isinstance(self.output_example, list) or isinstance(self.output_example, dict))
            else str(self.output_example)
        }
        for k in list(_prompt_object.keys()):
            if len(str(_prompt_object[k])) < 1:
                del _prompt_object[k]
        for k, v in kwargs.items():
            if not isinstance(k, str):
                k = str(k)
            if not isinstance(v, str):
                v = str(v)
            _prompt_object = json_replace(_prompt_object, f'{_VAR_LEFT}{k}{_VAR_RIGHT}', v)
        json_str = json.dumps(_prompt_object, ensure_ascii=False)
        _vars_left_to_fill = _VAR_PATTERN.findall(json_str)
        if len(_vars_left_to_fill) > 0:
            _vars_str = ', '.join([f'"{_}"' for _ in _vars_left_to_fill])
            raise TypeError(f'render() missing {len(_vars_left_to_fill)} required arguments: {_vars_str}')
        if markdown:
            return json_to_markdown(json_data=_prompt_object)
        return json_str



from abc import abstractmethod
from typing_extensions import Any


class BaseTemplate:
    def __init__(self):
        super().__init__()
        self._context: Any = ''
        self._role: Any = ''
        self._instruction: Any = ''
        self._objective: Any = ''
        self._capability: Any = ''
        self._constraint: Any = ''
        self._output_dtype: Any = ''
        self._output_format: Any = ''
        self._output_example: Any = ''
        self._output_language: Any = ''

    @abstractmethod
    def render(self, parse_markdown: bool) -> str: ...

    @property
    def context(self):
        return self._context

    @context.setter
    def context(self, value: Any):
        self._context = value

    @property
    def role(self):
        return self._role

    @role.setter
    def role(self, value: Any):
        self._role = value

    @property
    def instruction(self):
        return self._instruction

    @instruction.setter
    def instruction(self, value: Any):
        self._instruction = value

    @property
    def objective(self):
        return self._objective

    @objective.setter
    def objective(self, value: Any):
        self._objective = value

    @property
    def capability(self):
        return self._capability

    @capability.setter
    def capability(self, value: Any):
        self._capability = value

    @property
    def constraint(self):
        return self._constraint

    @constraint.setter
    def constraint(self, value: Any):
        self._constraint = value

    @property
    def output_dtype(self):
        return self._output_dtype

    @output_dtype.setter
    def output_dtype(self, value: Any):
        self._output_dtype = value

    @property
    def output_format(self):
        return self._output_format

    @output_format.setter
    def output_format(self, value: Any):
        self._output_format = value

    @property
    def output_example(self):
        return self._output_example

    @output_example.setter
    def output_example(self, value: Any):
        self._output_example = value

    @property
    def output_language(self):
        return self._output_language

    @output_language.setter
    def output_language(self, value: Any):
        self._output_language = value
